/**
 *
 * Copyright (c) 2001, Frank Bu
 *
 * project: LSystem
 * modul:   Test
 * version: $Revision: 1.2 $ $Name:  $
 *
 * CppUnit Testklasse fr Expression.
 */

// disable "identifier was truncated to 255 characters in the debug information"
#pragma warning(disable : 4786)

#include "test.h"
#include "TestEvaluator.h"

#include <string>
#include <math.h>
#include <iostream>

#include <Evaluator.h>

using namespace std;


void TestEvaluator::testAddAndEval()
{
	Evaluator e;
	e.addAction(new NumberAction(1));
	e.addAction(new NumberAction(2));
	e.addAction(new NumberAction(3));
	e.addAction(new MulAction());
	e.addAction(new AddAction());
	assert(e.eval() == (1+2*3));
}


void TestEvaluator::testVariable()
{
	Evaluator e;
	e.addAction(new NumberAction(1));
	e.addAction(new NumberAction(2));
	e.addAction(new VariableAction(&e, "test"));
	e.addAction(new MulAction());
	e.addAction(new AddAction());
	e.setVariable("test", 3);
	assert(e.eval() == (1+2*3));
}


double myMax(double op1, double op2)
{
	return op1 > op2 ? op1 : op2;
}


double mySqrt(double op)
{
	return sqrt(op);
}


double one()
{
	return 1;
}


void TestEvaluator::testFunction()
{
	Evaluator e;
	e.setFunction("one", one);
	e.setFunction("sqrt", mySqrt);
	e.setFunction("max", myMax);
	e.addAction(new NoArgumentFunctionAction(&e, "one"));
	assert(e.eval() == 1);
	e.removeAllActions();
	e.addAction(new NumberAction("81"));
	e.addAction(new OneArgumentFunctionAction(&e, "sqrt"));
	assert(e.eval() == 9);
	e.removeAllActions();
	e.addAction(new NumberAction("4"));
	e.addAction(new NumberAction("3"));
	e.addAction(new TwoArgumentsFunctionAction(&e, "max"));
	assert(e.eval() == 4);
}


Test *TestEvaluator::suite ()
{
	TestSuite *testSuite = new TestSuite("TestExpression");

    testSuite->addTest(new TestCaller <TestEvaluator> ("testAddAndEval", testAddAndEval));
    testSuite->addTest(new TestCaller <TestEvaluator> ("testVariable", testVariable));
    testSuite->addTest(new TestCaller <TestEvaluator> ("testFunction", testFunction));

	return testSuite;
}
